%% Problem5
% INPUT
% a, b, c - Points in R^2
% p, q - Vectors specifying a line G
% OUTPUT
% s, t - Vectors specifying the line of reflection

function [s, t] = Problem5(a1, a2, a3, p, q)
    
    % Determine the centroid of the triangle
    c = (a1+a2+a3)/3;
    
    % Calculate the distance between a and G
    r = norm(a1-c);
    centre = a1;
    intersection = Problem3NoPlot(r, centre, p, q);
    
    % Check whether there is a solution
    if isempty(intersection)
        disp('Such a reflection line does not exist');
        s = [];
        t = [];
        return
    else
       % Computing the parameters of the relfection line
       nt = intersection-c;
       t = [-nt(2); nt(1)];
       s = a1;
       
       % Plotting
       % Plot the line of reflection 
       x = [a1(1)-2*r; a1(1)+2*r];
       y = x*t(2)/t(1)-t(2)*s(1)/t(1)+s(2);
       plot(x, y, 'k--');

       % Plotting the circle
       phi = 0:pi/50:2*pi;
       axis equal
       hold on
       x = r * cos(phi) + centre(1);
       y = r * sin(phi) + centre(2);
       plot(x, y, 'k');
       
       % Plotting the triangle
       plot(a1(1), a1(2),'bo','LineWidth',1.5);
       plot(a2(1), a2(2),'bo','LineWidth',1.5);
       plot(a3(1), a3(2),'bo','LineWidth',1.5);
       plot([a1(1); a2(1)], [a1(2); a2(2)],'b');
       plot([a1(1); a3(1)], [a1(2); a3(2)],'b');
       plot([a3(1); a2(1)], [a3(2); a2(2)],'b');
       
       % Plot the centroid
       plot(c(1), c(2),'ro','LineWidth',1.5);
       % Plot the reflection
       plot([c(1); intersection(1)], [c(2); intersection(2)], 'g--');
       plot(intersection(1), intersection(2), 'go', 'LineWidth', 1.5);

       
       % Plot the line G
       xG= [a1(1)-2*r;a1(1)+2*r];
       yG = xG*q(2)/q(1)-q(2)*p(1)/q(1)+p(2);
       plot(xG, yG, 'r--');
       legend('reflection line','circle','a1','a2','a3',...
           'a1a2','a1a3','a3a2','centroid','reflection',...
           'intersection','line G')
    end
end


function [intersection] = Problem3NoPlot(r, centre, p, q)

% Coefficients for the governing quadratic polynomial, which appease the
% form the function zerosquadpol expects.
alpha = (2*(q(1)*(p(1)-centre(1))+q(2)*(p(2)-centre(2))) ) / norm(q, 2)^2;
beta = ((p(1)-centre(1))^2+(p(2)-centre(2))^2-r^2) / norm(q, 2)^2;

% The roots can then be computed by
tau = zerosquadpolstab(alpha, beta);

% The case when intersections exist
if isempty(tau) == 0   
    % Take either of the intersection points
    intersection = p+q*tau(1);
    
% The case when intersection does not exist
else
    intersection = [];
end

end


